// feedpatt.cpp - plots efficiency for convential parabolic dishes
//                from feed patterns
// Version 1.0    16 July 1997
//
// Paul Wade N1BWT copyright 1997
//
//      Any licensed radio amateur may use this program without 
//      charge; all other persons must send $73 to the 
//      ARRL Foundation, 225 Main St., Newington, CT 06111

//The source file, FEEDPATT.CPP, was compiled using Borland C++
//version 4.52.  Any part of it may be modified or used as part
//of another program by any amateur radio operator, provided
//that credit is given and provided there is no charge for
//its use.

//Send problems, suggestions, and comments to Paul Wade, N1BWT.
// n1bwt@qsl.net
//Expect support commensurate with the cost of the program!

// 

#include <dos.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <math.h>
#include <strstrea.h>
#include <ctype.h>

#include <iostream.h>
#include <string.h>
#include <complex.h>

#include "ps.h"

const char nl = '\n';
const double crad = 57.29577951 ;
const int max_ang = 181;
//const int ang_increment = 1;
const int LINESIZE = 81;

extern unsigned _stklen = 32000;


class feedpatt 
{
public:
  feedpatt();
  int npatt;
  int in_count;
  int colorplot;

  void banner(ostream&);
  void indata();
  void getfname(const char *fname);
  void fdata_in(const char *fname);
  void fpdata_in(const char *fname);
  void spline();
//  void spline(double x[], double y[], int j);
//  void splint(double xa[], double ya[], int n, double x, double &y);
  void interpolate();
  void power_to_voltage();
  void average_power_to_voltage();
  void integrate_pattern();
//  void getofname(const char *ofname);
  void PSfileheader(const char *ofname);
  void dplot(const char *ofname);
  void eplot(const char *ofname);

  double in_ang[max_ang];
  double in_dB[max_ang];
  double in_ratio[max_ang];
  double y2[max_ang];
  double Ep[max_ang];
  double Hp[max_ang];
  double vfeed[max_ang];
  double refl_dia ;
  double feed_dia ;

  double f_over_D[max_ang];
  double efficiency[max_ang];
  double illum_eff[max_ang];
  double spill_eff[max_ang];
  double blockage_eff[max_ang];


  
  int incount() {return in_count;};
  
private:
  void SkipRestLine(ifstream &file);
};


feedpatt::feedpatt()           // Constructor

{ 
   npatt = 0;
   in_count = 0;
   colorplot = 0;
   
   refl_dia  = 0.0;
   feed_dia  = 0.0;
   
   for (int i = 0; i < max_ang; i++)
     {
     in_ang[i] = 0.0;
     in_dB[i] = 0.0;
     in_ratio[i] = 0.0;
     y2[i] = 0.0;
     Ep[i] = 0.0;
     Hp[i] = 0.0;
     vfeed[i] = 0.0;
  
     f_over_D[i] = 0.0;
     efficiency[i] = 0.0;
     illum_eff[i] = 0.0;
     spill_eff[i] = 0.0;
     blockage_eff[i] = 0.0;
     }
}

void feedpatt::banner(ostream& ostrm)
{  /* a banner */
        
    ostrm << "\n\n\n\n";
    ostrm << "             ***************** \n";
    ostrm << "             *               * \n";
    ostrm << "             *   FEEDPATT    * \n";
    ostrm << "             *  version 1.0  * \n";
    ostrm << "             *               * \n";
    ostrm << "             ***************** \n";
    ostrm << "\n       copyright 1997 Paul Wade N1BWT \n\n";
    ostrm << "      Any licensed radio amateur may use this program "
          << "without \n";
    ostrm << "      charge; all other persons must send $73 to the \n";
    ostrm << "      ARRL Foundation, 225 Main St., Newington, CT 06111";
    ostrm << "\n\n";
}


void feedpatt::indata()
   {
   cout << "This routine plots efficiency for convential parabolic dishes"
        << nl;
   cout << "from feed patterns of files which contain data in the form: " 
        << nl << nl;
   cout << "angle(degrees)  dB" << nl << nl;
   cout << "with one data pair per line." << nl;
   cout << "Angle increments are arbitrary, but should be monotonic." << nl;
   cout << "dB are dB below pattern maximum." << nl << nl << nl;
   
   cin.clear();
   cout << "Enter dish diameter in wavelengths: ";
   cin >> refl_dia ;
   
   cin.clear();
   cout << nl
        << "Enter feed (or feed support, if larger) diameter in wavelengths: ";
   cin >> feed_dia ;
   }

void feedpatt::SkipRestLine(ifstream &file)
   {
   char ch;
   do file.get(ch);while (!file.fail() && ch != '\n');
   }
   
void feedpatt::getfname(const char *fname)
   {
   ifstream file(fname, ios::in | ios::nocreate);
   if (file.fail())
      {
      cout << "Cannot open input file. \n";  exit(1);
      }
   file.close();
   }
   
void feedpatt::fdata_in(const char *fname)
   {
   int i = 0;
   in_count = 0;
   
   ifstream file(fname, ios::in | ios::nocreate);
   for (;;)
      {
      file >> in_ang[i] >> in_dB[i];
      SkipRestLine(file);
      if (file.fail()) break;
      in_ratio[i] = pow(10.0,( 0.1 * (-in_dB[i])));
      cout << i << "   " << in_ang[i] << "   " << in_dB[i]
           << "   " << in_ratio[i] <<  nl;
      i++;
      }   
   file.close();
   in_count = i;
   npatt++;
   }



   
void feedpatt::fpdata_in(const char *fname)
// start from P. 33 of Borland v4.5 Class Libraries Guide
// to parse an input file

   { 
   int i = 0;
   in_count = 0;
   char description[61];
      
   ifstream file(fname, ios::in | ios::nocreate);
   
   if (file) 
      {
      char inbuf[81];
//      int lineno = 0;
      
      // Want floats to print as fixed point
      cout.setf(ios::fixed, ios::floatfield);
   
      // Want floats to always have decimal point
      cout.setf(ios::showpoint);
    
      while (file.getline(inbuf,81))
         {
         // 'ins' is the string stream
         istrstream ins(inbuf,strlen(inbuf));
         
         int cpeek = ins.peek();
         if (cpeek >= '0' && cpeek <= '9')
            {
            ins >> in_ang[i] >> in_dB[i] >> ws;
            ins.getline(description,41);    // Linefeed not copied

            in_ratio[i] = pow(10.0,( 0.1 * (-in_dB[i])));

            cout << i << ": "
                 << in_ang[i] << '\t'
                 << setprecision(2) << in_dB[i] << '\t'
                 << setprecision(4) << in_ratio[i] << nl;
            i++;
            }
         else
            { 
            ins.getline(description,41);    // Linefeed not copied
            cout << description << "\n";
            }
         }
      }
   file.close();
   in_count = i;
   npatt++;

   }


void feedpatt::spline()

// void feedpatt::spline(double x[], double y[], int j)
//void Spline(double x[], double y[], int n, double yp1, double ypn, double y2[])

//  Given arrays x[0..n] and y[0..n] containing a tabulated function, i.e.,
//  y[i] = f(x[i] ), with x[1] < x[2} < ... < x[N] , and given values yp1
//  and ypn for the first derivative of the interpolating function at
//  points 1 and n, respectively, this routine returns an array y2[1..n]
//  that contains the second derivatives of the interpolating function at
//  the tabulated points x[i] . If yp1 and/or ypn are equal to 1 x10**30
//  or larger, the routine is signaled to set the corresponding boundary
//  condition for a natural spline, with zero second derivative on that
//  boundary.
    
    {
    int i,k;
    double p,qn,sig,un;
    double u[max_ang];           // inefficient fixed allocation for now
    double yp1, ypn;         // first derivatives of endpoints
    double x[max_ang];
    double y[max_ang];

//setup

	 int n = in_count - 1;    // array index is one less than count

// first derivatives of endpoints

    yp1 = (in_ratio[1] - in_ratio[0]) / (in_ang[1] - in_ang[0]);
    ypn = (in_ratio[n] - in_ratio[n-1]) / (in_ang[n] - in_ang[n-1]);

    
    for (i = 0; i <= n; i++)
        {
        x[i] = in_ang[i];
        y[i] = in_ratio[i];
        }


// start actual spline routine

    if (yp1 > 0.99e30)      // The lower boundary condition is set either to
        y2[0]=u[0]=0.0;    // be ``natural''
    else                    // or else to have a specified first derivative.
        {
        y2[0] = -0.5;
        u[0]=(3.0/(x[1]-x[0]))*((y[1]-y[0])/(x[1]-x[0])-yp1);
        }

//      This is the decomposition loop of the tridiagonal algorithm.
//      y2 and u are used for temporary storage of the decomposed factors.

    for (i=1;i!=n;i++)
        {
        sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
        p=sig*y2[i-1]+2.0;
        y2[i]=(sig-1.0)/p;
        u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
        u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
		  }

	 if (ypn > 0.99e30)   // The upper boundary condition is set
		  qn=un=0.0;     // either to be ``natural''
	 else                 // or else to have a specified first derivative.
		  {
		  qn=0.5;
		  un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
		  }

	 y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);


	 for (k=n-1;k>=0;k--)          // This is the backsubstitution loop
		  y2[k]=y2[k]*y2[k+1]+u[k]; // of the tridiagonal algorithm.


	 }


// void feedpatt::splint(double xa[], double ya[], int n, double x, double &y)

//      It is important to understand that the program spline is called
//      only once to process an entire tabulated function in arrays x i
//      and y i . Once this has been done, values of the interpolated
//      function for any value of x are obtained by calls (as many as
//      desired) to a separate routine splint (for ``spline
//      interpolation''): 

void Splint(double xa[], double ya[], double y2a[], int n, double x, double &y) 

//      Given the arrays xa[0..n] and ya[0..n], which tabulate a function
//      (with the xa i 's in order), and given the array y2a[0..n], which
//      is the output from spline above, and given a value of x, this
//      routine returns a cubic-spline interpolated value y. 

{  
    int klo,khi,k; 
    double h,b,a; 
	 const char nl = '\n';

//  We will find the right place in the table by means of
//  bisection. This is optimal if sequential calls to this
//  routine are at random values of x. If sequential calls
//  are in order, and closely spaced, one would do better
//  to store previous values of klo and khi and test if
//  they remain appropriate on the next call.

	 klo=0;
	 khi=n;
	 while (khi-klo > 1)
		  {
		  k=(khi+klo) >> 1;
		  if (xa[k] > x) khi=k;
		  else klo=k;
		  }
//  klo and khi now bracket the input value of x.

	 h=xa[khi]-xa[klo];

	 if (h == 0.0) cerr << "Bad xa input to routine splint" << nl ;
//    The xa's must be distinct.

	 a=(xa[khi]-x)/h;
	 b=(x-xa[klo])/h;

//      Cubic spline polynomial is now evaluated.
	 y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;

}

void feedpatt::interpolate()
   {
   double amplitude;
   int i;
   int n = in_count - 1;                //array index

   

	for ( i = 0; i <= 180; i++)
		{
		Splint(in_ang, in_ratio, y2, n, double(i), amplitude);

                if (amplitude < 0.0)
                   { 
                   if (fabs(amplitude) < 0.1)
                      amplitude = 0.000001;
                   else 
                      {
                      cerr << " Bogus interpolation at " << i << "degrees" << nl;
                      exit(1);
                      }
                   }

		if (npatt == 1)
                   Ep[i] = amplitude;
                else 
                   Hp[i] = amplitude;
                
		}
	}

void feedpatt::power_to_voltage()
// temp until average power is working
	{
        double tmpwr = 0.0;
//   ofstream dfile("v_diag.out");

// dfile << " in power_to_voltage()" << nl;
	for (int i = 0; i <= 180; i++)
		{
                tmpwr = -10.0 * log10(Ep[i]);
		vfeed[i] = pow(10.0,(-0.05 * tmpwr));
		}

	}


void feedpatt::average_power_to_voltage()
	{
        double tmpwr = 0.0;

	for (int i = 0; i <= 180; i++)
		{
                tmpwr = -10.0 * log10(0.5 * (Ep[i] + Hp[i]));
		vfeed[i] = pow(10.0,(-0.05 * tmpwr));
		}
	}


void feedpatt::integrate_pattern()
// takes a feed pattern and calculates efficiency
// for various f/D -- started with DISHFEED.BAS by W7PUA

	 {
	 int i;
	 int ang;
	 double in1[max_ang];
	 double in2[max_ang];
	 double denom;
	 double u;
	 double theta;
	 double num;
	 double deni;
	 double numb;
	 double K;
	 double tmp;
	 double edge_illum[max_ang];
	 double unblocked_eff;
	 double block_angle;
	 int block_deg;


//          now start integrating the pattern
	 denom = 0.0;
	 for (i = 1; i <= 180; i++)
		  {
		  u = 0.5 * (vfeed[i - 1] + vfeed[i]);


//        theta = pi * (double(i) - 0.5) / 180.0;
		  theta = (double(i) - 0.5) / crad;
                    
		  in1[i] = u * tan(0.5 * theta);          // half-angle
		  in2[i] = u * u * sin(theta);
		  denom = denom + in2[i];

		  }
//    denom = pi * denom / 180.0;
	 denom = denom /crad;
  
// initialize arrays
	 for (i = 0; i <= 180; i++)
		  {
		  edge_illum[i] = 0.0;
		  illum_eff[i] = 0.0;
		  spill_eff[i] = 0.0;
		  efficiency[i] = 0.0;
		  blockage_eff[i] = 0.0;
		  f_over_D[i] = 0.0;
		  }

	 for (ang = 20; ang <= 90; ang++ )
             {
             tmp = tan(((double)ang)/(2.0 * crad));
             f_over_D[ang] = 1.0 / (4.0 * tmp);

             block_angle = atan2(feed_dia,( 2.0 * refl_dia * f_over_D[ang] ));
             block_deg = int(block_angle * crad);
             num = 0.0;
             deni = 0.0;
             numb = 0.0;
             for (i = 1; i <= ang; i++)
		 {
		 num = num + in1[i];
		 deni = deni + in2[i];
                 if (i > block_deg)
                 numb = numb + in1[i];            // feed blockage
                 }
//        num = pi * num / 180.0;
//        deni = pi * deni / 180.0;
             num = num / crad;
             deni = deni / crad;
             numb = numb / crad;
// cerr << " integrating" << ang << "   " << num << "   " << numb << nl;


             K = 2.0 / (tmp * tmp);
//  summarize results
             if (vfeed[ang] > 0.0)
                edge_illum[ang] = 20.0 * log10(vfeed[ang]);
             else
                {
                cerr << " illegal log()"; 
                exit(1);
                }
             unblocked_eff = K * num * num / denom;
             illum_eff[ang] = K * num * num / deni;
             spill_eff[ang] = unblocked_eff / illum_eff[ang];
             efficiency[ang] = K * numb * numb / denom;
             blockage_eff[ang] = efficiency[ang] / (illum_eff[ang] * spill_eff[ang]);
             }

//  output to screen
         cout << nl << nl;
	 cout << "  Half   f/D       Edge     --------EFFICIENCIES-------------" << nl;
	 cout << " Angle            Illum     TOTAL    Illum     Spillover  Blockage" << nl;
         cout << nl;


// Want floats to print as fixed point
         cout.setf(ios::fixed, ios::floatfield);
   
// Want floats to always have decimal point
         cout.setf(ios::showpoint);



	 for (ang = 20; ang <= 90; ang = ang + 5)
		  {
		  cout << "  " << ang
                       << "    " << setprecision(3) << f_over_D[ang]
                       << "   " << setprecision(2) << edge_illum[ang]
                       << " dB    " << setprecision(2) << (100.0 * efficiency[ang])
                       << "%   " << setprecision(2) << (100.0 * illum_eff[ang])
                       << "%     " << setprecision(2) << (100.0 * spill_eff[ang])
                       << "%    " << setprecision(2) << (100.0 * blockage_eff[ang])
                       << "%" << nl;
		  }
	 }



void feedpatt::PSfileheader(const char *ofname)
   {
   ofstream ofile(ofname, ios::out);
  
   if (ofile.fail())
      {
      cout << "Cannot open output file. \n";  exit(1);
      }

    char ps_title[LINESIZE] = " ";

    cin.ignore(LINESIZE,'\n');
//    cin.clear();
    cout << nl << " Title line for output graph: ";
    cout.flush();
    cin.get(ps_title,LINESIZE,'\n');

    char color;
    cin.ignore(LINESIZE,'\n');
//    cin.clear();
    cout << nl << " Plot in color (Y or N)? ";
    cout.flush();
    cin.get(color);
    color = toupper(color);

    if (color == 'Y')
       colorplot = 1;


   ofile << "%!PS-Adobe-3.0 EPSF-3.0"  << nl;
   ofile << "%! PostScript \n";
   ofile << "%%BoundingBox: 60 60 550 715"  << nl;
   ofile << "%%Title: (" << ps_title << ") \n";
   ofile << "%%Creator: FEEDPATT by N1BWT" << nl;
   ofile << "%%DocumentNeededResources: font Times-Bold Helvetica-Bold Symbol"
         << nl;
   ofile << "%%+ font Times-BoldItalic" << nl;      
   ofile << "save"  << nl;

   ofile << "/mm { 2.834646 mul } def\n";
   ofile << "/in { 72 mul } def\n";

   ofile << "/Times-Bold findfont\n";
   ofile << "18 scalefont\n";
   ofile << "setfont\n";

//   ofile << "  2 in 9.5 in  moveto \n";
//   ofile << " (" << ps_title << ") show \n";

// magic to center the title
   ofile << " (" << ps_title << ") stringwidth pop" << nl;
   ofile << " 2 div neg 306 add 9.7 in moveto" << nl;
   ofile << " (" << ps_title << ") show \n";

   ofile << " (Figure 18) stringwidth pop" << nl;
   ofile << " 2 div neg 306 add 9.3 in moveto" << nl;
   ofile << " () show \n";
   ofile << " %(Figure X) show \n";


   ofile << "/Times-Bold findfont\n";
   ofile << "12 scalefont\n";
   ofile << "setfont\n";

   ofile << "  5.5 in 6.7 in  moveto \n";
   ofile << " (Dish diameter = "
         << setprecision(3) << refl_dia 
         << ") show" << nl;
//         << " wavelengths) show \n";

   ofile << "/Symbol findfont 12 scalefont setfont ( l) show " << nl;


   ofile << "/Times-Bold findfont\n";
   ofile << "12 scalefont\n";
   ofile << "setfont\n";

   ofile << "  5.5 in 6.5 in  moveto \n";
   ofile << " (Feed diameter = "
         << setprecision(3) << feed_dia 
         << ") show" << nl;
//         << " wavelengths) show \n";

   ofile << "/Symbol findfont 12 scalefont setfont ( l) show " << nl;
   

   ofile.close();
	npatt++;
   }



void feedpatt::dplot(const char *ofname)
//dplot.cpp - to draw feed patterns from a pattern file
// 6/27/97
// started from:
//mplot.cpp - 


    {
//    char cmnt[256];

    int circles = 4;  // 4 = 0, -10, -20, -30 db circles
    float height = 30; // scale 

    ofstream ofile(ofname, ios::app);


	 Point a;
         int j;
	 float x,y,m,ang;
	 PS_lineto b;

//	 ofile << "%! PostScript \n";
//	 ofile << "/mm { 2.834646 mul } def\n";
//	 ofile << "/in { 72 mul } def\n";

	 ofile << "gsave\n";

	 ofile << "110 mm 202 mm translate\n";

// dB circles
         ofile << "%start dB circles" << nl;
	 ofile << "gsave\n";

	 double xcircles = double(circles);

	 for ( j=1; j <= circles; j++ )
		 {
		 double xj = double(j);


		 ofile << "newpath\n" ;
		 ofile << PS_moveto((height * xj / xcircles),0) << "\n";

		 for (int deg = 0; deg <= 360; deg++)
			 {
			 ang = deg / crad;
			 m = height * xj / xcircles;

			 complex z = polar(m,ang);

			 x = real(z);
			 y = imag(z);

			 ofile << PS_lineto(x,y) << "\n";
			 }

		 ofile << "stroke\n" ;
		 }
	 ofile << "grestore\n" ;

// line to split planes
         ofile << PS_moveto(height,0) << "\n";
         ofile << PS_lineto(-height,0) << "\n";
         ofile << "stroke\n" ;

// label planes

	 ofile << "/Helvetica-Bold findfont\n";
	 ofile << "10 scalefont\n";
	 ofile << "setfont\n";

	 ofile << "-4 mm 25 mm moveto \n";
	 ofile << " (E-plane ) show \n";

	 ofile << "-4 mm -27 mm moveto \n";
	 ofile << " (H-plane ) show \n";

// scale circles

	 ofile << "/Helvetica-Bold findfont\n";
	 ofile << "8 scalefont\n";
	 ofile << "setfont\n";

	 ofile << "-36.5 mm 1 mm moveto \n";
	 ofile << " (0 dB) show \n";

	 ofile << "-27 mm 1 mm moveto \n";
	 ofile << " (-10 ) show \n";

	 ofile << "-19.5 mm 1 mm moveto \n";
	 ofile << " (-20 ) show \n";

	 ofile << "-12 mm 1 mm moveto \n";
	 ofile << " (-30 ) show \n";



//        m = pow(m,n);
// hack to overcome bogus Borland math library
//log scale

//        m *= 1000.0;    // for circles =3


         ofile << "%start polar plots" << nl;

         ofile << "gsave\n" ;
         ofile << "2 setlinewidth" << nl;
           
          double mscale = pow(10.0,xcircles);

          for ( j = 0; j <= 1; j++)
              {        
              for (int deg = 0; deg <= 180; deg++)
		 {
                 if (j == 0)
		    {
                    m = mscale * Ep[deg];  // E-plane power plot
                    ang = deg/crad;
                    }
                 else
                    {
		    m = mscale * Hp[deg];  // H-plane power plot
                    ang = -deg/crad;
                    }
                    
                 if (m > 1.0)   // cerr << " Error, m = " << m ;
		    {
//            m = n * (log10(m) - 3.0 );  //  log (   m = pow(m,n) );
//	    m += 3.0;

                    m = log10(m);  //  log (   m = pow(m,n) );
                    if (m > 0.0)   // cerr << " Error, m = " << m ;
                       {
                       m = m * height/xcircles;
//                       ang = deg/crad;

                       complex z = polar(m,ang);

                       x = real(z);
                       y = imag(z);

                       if (deg == 0)
                          ofile << PS_moveto(x,y) << "\n";
                       else
                          ofile << PS_lineto(x,y) << "\n";
                       }
                    }
                 }
              
	 ofile << "stroke\n" ;
             }
	 ofile << "grestore\n" ;


    
// put some text in the picture


	 ofile << "/Helvetica-Bold findfont\n";
	 ofile << "12 scalefont\n";
	 ofile << "setfont\n";


	 ofile << "-75 mm -30 mm moveto \n";
	 ofile << " (N1BWT 1997) show \n";



	 ofile << "grestore\n" ;

//    ofile << "showpage\n" ;

    ofile.close();
    
//  return 0;

    }

void feedpatt::eplot(const char *ofname)
//eplot.cpp - to draw efficiency plots 
// 6/30/97
// started from:
//mplot.cpp - 


    {
//    char cmnt[256];


    ofstream ofile(ofname, ios::app);



	 Point a;
	 float x,y;
         int ang;
	 PS_lineto b;
         float eheight = 125.0;
         float ewidth  = 140.0;
         
         float escale = 4.0 * ewidth / 3.0;

//	 ofile << "%! PostScript \n";
//	 ofile << "/mm { 2.834646 mul } def\n";
//	 ofile << "/in { 72 mul } def\n";

	 ofile << "gsave\n";
	 ofile << "40 mm 37 mm translate\n";

// outline graph
         ofile << "%start drawing graph" << nl;

	 ofile << "gsave\n";
         ofile << "2 setlinewidth" << nl;

         ofile << PS_moveto(0.0,0.0) << nl ;
         ofile << PS_lineto(ewidth,0.0) << nl ;
         ofile << PS_lineto(ewidth,eheight) << nl ;
         ofile << PS_lineto(0.0,eheight) << nl ;
         ofile << PS_lineto(0.0,0.0) << nl ;

	 ofile << "stroke\n" ;

// scale graph

	 ofile << "/Helvetica-Bold findfont\n";
	 ofile << "10 scalefont\n";
	 ofile << "setfont\n";

// horizontal divisions
        ofile << "1 setlinewidth" << nl;

         for (int j = 3; j <= 9; j++)
             {
             x = (float(j) - 2.5) *  0.1  * escale ;
             ofile << setprecision(4) << PS_moveto(x,0.0) << nl ;
             ofile << PS_lineto(x,eheight) << " stroke" << nl ;
             ofile << PS_moveto(x - 2,-4) << nl ;
             ofile << " (0." << setprecision(1) << j << ") show \n";
             }
// ends
             ofile << PS_moveto( - 2,-4) << nl ;
             ofile << " (0.25) show \n";
             ofile << PS_moveto(( escale - 2),-3) << nl ;
             ofile << " (1.0) show \n";



// vertical divisions
        ofile << "1 setlinewidth" << nl;

        for (j = 1; j <= 9; j++)
            {
             y = j * 0.1 * eheight;
             ofile << setprecision(4) << PS_moveto(0.0,y) << nl ;
             ofile << PS_lineto(ewidth,y) << " stroke" << nl ;
             ofile << PS_moveto( - 6,y - 1) << nl ;

//             ofile << " (0." << setprecision(1) << j << ") show \n";
// in percent
             ofile << " (" << setprecision(2) << (j * 10) << ") show \n";
            }
            
// dB tics on right side

        ofile << "1 setlinewidth" << nl;

        for (j = 1; j <= 8; j++)
            {
             y = eheight * pow(10.0,(-0.1 * j));
             ofile << setprecision(4) << PS_moveto(ewidth,y) << nl ;
             ofile << PS_lineto(ewidth + 1.0,y) << " stroke" << nl ;
             ofile << PS_moveto(ewidth + 2.0,y - 1.0) << nl ;
             ofile << " (" << setprecision(1) << j << " dB) show \n";
            }
  

	ofile << "grestore\n" ;


// plot total efficiency data
        ofile << "%start efficiency curve" << nl;

        ofile << "gsave\n";

        if (colorplot)
           ofile << "1 0 0 setrgbcolor" << nl;
        else
           ofile << "% 1 0 0 setrgbcolor" << nl;

        ofile << "2 setlinewidth" << nl;

        ofile << setprecision(4)
               << PS_moveto((escale * (f_over_D[90] - 0.25)),
                                 (eheight * efficiency[90])) << nl;

        for (ang = 90; ang >= 28; ang--)
            ofile << PS_lineto((escale * (f_over_D[ang] - 0.25)),
                                (eheight * efficiency[ang])) << nl;

                
	 ofile << "stroke\n" ;
//legend
        ofile  << PS_moveto( escale * 0.02, eheight * 0.95) << nl;
        ofile  << PS_lineto( escale * 0.08, eheight * 0.95) << nl;
	 ofile << "stroke\n" ;
         

       ofile << "/Times-Bold findfont\n";
       ofile << "12 scalefont\n";
       ofile << "setfont\n";

        ofile  << PS_moveto( escale * 0.09, eheight * 0.94) << nl;
        ofile << "(MAX Possible Efficiency) show" << nl;


        ofile  << PS_moveto( escale * 0.47, eheight * 0.94) << nl;
        ofile << "(AFTER LOSSES:) show" << nl;

       ofile << "/Times-BoldItalic findfont\n";
       ofile << "12 scalefont\n";
       ofile << "setfont\n";


        ofile  << PS_moveto( escale * 0.09, eheight * 0.86) << nl;
        ofile << "(REAL WORLD at least 15% lower) show" << nl;


	 ofile << "grestore\n" ;

// plot illumination loss data
        ofile << "%start illumination curve" << nl;

        ofile << "gsave\n";

        if (colorplot)
           ofile << "0 1 0 setrgbcolor" << nl;
        else
           ofile << "% 0 1 0 setrgbcolor" << nl;

        ofile << "1 setlinewidth" << nl;
        ofile << "[6 5] 0 setdash" << nl;

        ofile << setprecision(4)
               << PS_moveto((escale * (f_over_D[90] - 0.25)),
                                 (eheight * (1.0 - illum_eff[90]))) << nl;

        for (ang = 90; ang >= 30; ang = ang - 5)
            ofile << PS_lineto((escale * (f_over_D[ang] - 0.25)),
                                 (eheight * (1.0 - illum_eff[ang]))) << nl;

                

	 ofile << "stroke\n" ;
//legend
        ofile  << PS_moveto( escale * 0.47, eheight * 0.87) << nl;
        ofile  << PS_lineto( escale * 0.53, eheight * 0.87) << nl;
	 ofile << "stroke\n" ;
         

   ofile << "/Times-Bold findfont\n";
   ofile << "12 scalefont\n";
   ofile << "setfont\n";

        ofile  << PS_moveto( escale * 0.56, eheight * 0.86) << nl;
        ofile << "(Illumination ) show" << nl;


	 ofile << "grestore\n" ;

// plot spillover loss data
        ofile << "%start spillover curve" << nl;

        ofile << "gsave\n";

        if (colorplot)
           ofile << "0 0 1 setrgbcolor" << nl;
        else
           ofile << "% 0 0 1 setrgbcolor" << nl;

        ofile << "1 setlinewidth" << nl;
        ofile << "[2 2] 0 setdash" << nl;

        ofile << setprecision(4)
               << PS_moveto((escale * (f_over_D[90] - 0.25)),
                                 (eheight * (1.0 - spill_eff[90]))) << nl;

        for (ang = 90; ang >= 30; ang = ang - 5)
            ofile << PS_lineto((escale * (f_over_D[ang] - 0.25)),
                                 (eheight * (1.0 - spill_eff[ang]))) << nl;

                

	 ofile << "stroke\n" ;
         
//legend
        ofile  << PS_moveto( escale * 0.47, eheight * 0.83) << nl;
        ofile  << PS_lineto( escale * 0.53, eheight * 0.83) << nl;
	 ofile << "stroke\n" ;

   ofile << "/Times-Bold findfont\n";
   ofile << "12 scalefont\n";
   ofile << "setfont\n";


        ofile  << PS_moveto( escale * 0.56, eheight * 0.82) << nl;
        ofile << "(Spillover ) show" << nl;

	 ofile << "grestore\n" ;

// plot blockage loss data
        ofile << "%start blockage curve" << nl;

        ofile << "gsave\n";

        if (colorplot)
           ofile << "1 0.5 0.5 setrgbcolor" << nl;
        else 
           ofile << "% 1 0.5 0.5 setrgbcolor" << nl;

        ofile << "1 setlinewidth" << nl;
        ofile << "[6 3 2 3] 0 setdash" << nl;

        ofile << setprecision(4)
               << PS_moveto((escale * (f_over_D[90] - 0.25)),
                                 (eheight * (1.0 - blockage_eff[90]))) << nl;

        for (ang = 90; ang >= 30; ang = ang - 5)
            ofile << PS_lineto((escale * (f_over_D[ang] - 0.25)),
                                 (eheight * (1.0 - blockage_eff[ang]))) << nl;


	 ofile << "stroke\n" ;
         
//legend
        ofile  << PS_moveto( escale * 0.47, eheight * 0.77) << nl;
        ofile  << PS_lineto( escale * 0.53, eheight * 0.77) << nl;
	 ofile << "stroke\n" ;

   ofile << "/Times-Bold findfont\n";
   ofile << "12 scalefont\n";
   ofile << "setfont\n";

        ofile  << PS_moveto( escale * 0.56, eheight * 0.76) << nl;
        ofile << "(Feed Blockage ) show" << nl;

	 ofile << "grestore\n" ;


    
// put some text in the picture


	 ofile << "/Helvetica-Bold findfont\n";
	 ofile << "18 scalefont\n";
	 ofile << "setfont\n";


	 ofile << " 45 mm -12 mm moveto \n";
	 ofile << " (Parabolic Dish f/D) show \n";

      ofile << "gsave" << nl ;
	 ofile << " -10 mm 40 mm moveto \n";
         ofile << "90 rotate" << nl;
	 ofile << " (Efficiency %) show \n";
        ofile << "grestore\n" ;


	 ofile << "grestore\n" ;
	 ofile << "grestore\n" ;

    ofile << "restore\n" ;
    ofile << "showpage\n" ;
    ofile << "%%EOF\n" ;

    ofile.close();
    
    }


///***************************************

int main(int argc, char **argv)
   {
   int args;

   char fname[50];
   
   cerr << ( argc - 1) << " files specified " << "\n";

   feedpatt g;
   
   g.banner(cout);
   
   g.indata();        
   
   args = argc - 1;

//cerr << "args = " << args << nl;

   if (args > 0)
      {
      strcpy(fname,argv[1]);
      args-- ;
      }
   else
      {
      cin.clear();
      cout << "Input file for E-plane pattern: ";
      cin >> setw(50) >> fname;
      }

   g.getfname(fname);
   g.fpdata_in(fname);
// the input routine increments npatt


   g.spline();
   g.interpolate();


//cerr << "args = " << args << nl;

   if (args > 0)
      {
      strcpy(fname,argv[2]);
      args-- ;
      }
   else
      {
      cin.clear();
      cout << nl << nl << "Input file for H-plane pattern: ";
      cin >> setw(50) >> fname;
      }

   g.getfname(fname);
   g.fpdata_in(fname);
// the input routine increments npatt

   g.spline();
   g.interpolate();


   g.average_power_to_voltage();

   g.integrate_pattern();

//cerr << "args = " << args << nl;

   char ofname[50];

   if (args > 0)
      {
      strcpy(ofname,argv[3]);
      args-- ;
      }
   else
      {
      cin.clear();
      cout << nl << "Output file (should be .EPS ): ";
      cin >> setw(50) >> ofname;
      }

   g.PSfileheader(ofname);

   g.dplot(ofname);

   g.eplot(ofname);

   cout << nl << nl
        << " Use Ghostscript to view or print output file." << nl;
        
   return 0;
   }
